@@ -31,6 +31,68 @@
 # missnumspec = number of missing species    
 # methode = the method used in the numerical solving of the set of the ode's
 
+
+
+#' Loglikelihood for diversity-dependent diversification models with a shift in
+#' the parameters at time t = tshift
+#' 
+#' This function computes loglikelihood of a diversity-dependent
+#' diversification model for a given set of branching times and parameter
+#' values where the parameters are allowed to shift at time t = tshift
+#' 
+#' 
+#' @param pars1 Vector of parameters: \cr \cr \code{pars1[1]} corresponds to
+#' lambda (speciation rate) before the shift \cr \code{pars1[2]} corresponds to
+#' mu (extinction rate) before the shift \cr \code{pars1[3]} corresponds to K
+#' (clade-level carrying capacity) before the shift \cr \code{pars1[4]}
+#' corresponds to lambda (speciation rate) after the shift \cr \code{pars1[5]}
+#' corresponds to mu (extinction rate) after the shift \cr \code{pars1[6]}
+#' corresponds to K (clade-level carrying capacity) after the shift \cr
+#' \code{pars1[7]} corresponds to tshift (the time of shift)
+#' @param pars2 Vector of model settings: \cr \cr \code{pars2[1]} sets the
+#' maximum number of species for which a probability must be computed.  This
+#' must be larger than 1 + missnumspec + length(brts). \cr \cr \code{pars2[2]}
+#' sets the model of diversity-dependence: \cr - \code{pars2[2] == 1} linear
+#' dependence in speciation rate with parameter K (= diversity where speciation
+#' = extinction)\cr - \code{pars2[2] == 1.3} linear dependence in speciation
+#' rate with parameter K' (= diversity where speciation = 0)\cr -
+#' \code{pars2[2] == 2} exponential dependence in speciation rate with
+#' parameter K (= diversity where speciation = extinction)\cr - \code{pars2[2]
+#' == 2.1} variant of exponential dependence in speciation rate with offset at
+#' infinity\cr - \code{pars2[2] == 2.2} 1/n dependence in speciation rate\cr -
+#' \code{pars2[2] == 2.3} exponential dependence in speciation rate with
+#' parameter x (= exponent)\cr - \code{pars2[2] == 3} linear dependence in
+#' extinction rate \cr - \code{pars2[2] == 4} exponential dependence in
+#' extinction rate \cr - \code{pars2[2] == 4.1} variant of exponential
+#' dependence in extinction rate with offset at infinity\cr - \code{pars2[2] ==
+#' 4.2} 1/n dependence in extinction rate\cr\cr \code{pars2[3]} sets the
+#' conditioning: \cr - \code{pars2[3] == 0} no conditioning \cr -
+#' \code{pars2[3] == 1} conditioning on non-extinction of the phylogeny \cr -
+#' \code{pars2[3] == 2} conditioning on non-extinction of the phylogeny and on
+#' the total number of extant taxa (including missing species) \cr \cr
+#' \code{pars2[4]} sets whether the likelihood is for the branching times (0)
+#' or the phylogeny (1) \cr \cr \code{pars2[5]} sets whether the parameters and
+#' likelihood should be shown on screen (1) or not (0) \cr \cr \code{pars2[6]}
+#' sets whether the first data point is stem age (1) or crown age (2)
+#' @param brts A set of branching times of a phylogeny, all positive
+#' @param missnumspec The number of species that are in the clade but missing
+#' in the phylogeny
+#' @param methode The method used to solve the master equation, default is
+#' 'analytical' which uses matrix exponentiation; alternatively numerical ODE
+#' solvers can be used, such as 'lsoda' or 'ode45'. These were used in the
+#' package before version 3.1.
+#' @return The loglikelihood
+#' @author Rampal S. Etienne & Bart Haegeman
+#' @seealso \code{\link{dd_SR_ML}}, \code{\link{dd_loglik}},
+#' \code{\link{dd_KI_loglik}}
+#' @references - Etienne, R.S. et al. 2012, Proc. Roy. Soc. B 279: 1300-1309,
+#' doi: 10.1098/rspb.2011.1439 \cr - Etienne, R.S. & B. Haegeman 2012. Am. Nat.
+#' 180: E75-E89, doi: 10.1086/667574
+#' @keywords models
+#' @examples
+#' dd_SR_loglik(pars1 = c(0.2,0.1,50,0.2,0.1,70,5), pars2 = c(100,1,1,1,0,2),
+#'    brts = 1:10, missnumspec = 0) 
+#' @export dd_SR_loglik
 dd_SR_loglik = function(pars1,pars2,brts,missnumspec,methode = 'analytical')
 {
    if(methode == 'analytical')
@@ -167,7 +229,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
               for(k in (S + 2 - soc):(kshift + 1))
               {
                  k1 = k + (soc - 2)
-                 y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+                 y = deSolve::ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
                  probs = y[2,2:(lx+2)]
                  if(k > soc)
                  {
@@ -185,9 +247,9 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            }
            k = kshift
            k1 = k + (soc - 2)
-           y = ode(probs,-c(brts[k],tshift),dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+           y = deSolve::ode(probs,-c(brts[k],tshift),dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            probs = y[2,2:(lx+2)]
-           y = ode(probs,-c(tshift,brts[k-1]),dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+           y = deSolve::ode(probs,-c(tshift,brts[k-1]),dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            probs = y[2,2:(lx+2)]
            if(k > soc)
            {
@@ -203,7 +265,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            for(k in (kshift-1):2)
            {
               k1 = k + (soc - 2)
-              y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+              y = deSolve::ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
               probs = y[2,2:(lx+2)]
               if(k > soc)
               {
@@ -246,16 +308,16 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
              probsn = rep(0,lx + 1)
              probsn[S + missnumspec + 1] = 1
              TT = max(1,1/abs(la - mu),1/abs(la2 - mu2)) * 1E+8 * max(abs(brts)) # make this more efficient later
-             y = ode(probsn,c(0,-tshift),dd_loglik_bw_rhs,c(pars1[4:6],0,ddep),rtol = reltol,atol = abstol, method = methode)
+             y = deSolve::ode(probsn,c(0,-tshift),dd_loglik_bw_rhs,c(pars1[4:6],0,ddep),rtol = reltol,atol = abstol, method = methode)
              probsn = y[2,2:(lx+2)]
-             y = ode(probsn,c(-tshift,TT),dd_loglik_bw_rhs,c(pars1[1:3],0,ddep),rtol = reltol,atol = abstol, method = methode)
+             y = deSolve::ode(probsn,c(-tshift,TT),dd_loglik_bw_rhs,c(pars1[1:3],0,ddep),rtol = reltol,atol = abstol, method = methode)
              logliknorm = log(y[2,lx + 2])
              if(soc == 2)
              {
                 probsn = rep(0,lx + 1)
                 probsn[1:lx] = probs[1:lx]
                 probsn = c(flavec(ddep,la,mu,K,0,lx,1,n0),1) * probsn # speciation event
-                y = ode(probsn,c(max(abs(brts)),TT),dd_loglik_bw_rhs,c(pars1[1:3],1,ddep),rtol = reltol,atol = abstol, method = methode)
+                y = deSolve::ode(probsn,c(max(abs(brts)),TT),dd_loglik_bw_rhs,c(pars1[1:3],1,ddep),rtol = reltol,atol = abstol, method = methode)
                 logliknorm = logliknorm - log(y[2,lx + 2])
              }
           }
@@ -268,7 +330,7 @@ if(pars2[5] == 1)
     s1 = sprintf('Parameters: %f %f %f %f %f %f %f, ',pars1[1],pars1[2],pars1[3],pars1[4],pars1[5],pars1[6],pars1[7])
     s2 = sprintf('Loglikelihood: %f',loglik)
     cat(s1,s2,"\n",sep = "")
-    flush.console()
+    utils::flush.console()
 }
 loglik = as.numeric(loglik)
 if(is.nan(loglik) | is.na(loglik))
@@ -402,7 +464,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
               for(k in (S + 2 - soc):(kshift + 1))
               {
                  k1 = k + (soc - 2)
-                 #y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+                 #y = deSolve::ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
                  #probs = y[2,2:(lx+2)]
                  probs = dd_loglik_M_bw(pars1[4:6],lx,k1,ddep,tt = abs(brts[k - 1] - brts[k]),probs[1:lx])
                  probs = c(probs,0) 
@@ -423,11 +485,11 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            }
            k = kshift
            k1 = k + (soc - 2)
-           #y = ode(probs,-c(brts[k],tshift),dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+           #y = deSolve::ode(probs,-c(brts[k],tshift),dd_loglik_bw_rhs,c(pars1[4:6],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            #probs = y[2,2:(lx+2)]
            probs = dd_loglik_M_bw(pars1[4:6],lx,k1,ddep,tt = abs(tshift - brts[k]),probs[1:lx])
            probs = c(probs,0)            
-           #y = ode(probs,-c(tshift,brts[k-1]),dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+           #y = deSolve::ode(probs,-c(tshift,brts[k-1]),dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
            #probs = y[2,2:(lx+2)]
            probs = dd_loglik_M_bw(pars1[1:3],lx,k1,ddep,tt = abs(brts[k - 1] - tshift),probs[1:lx])
            probs = c(probs,0) 
@@ -446,7 +508,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
            for(k in (kshift-1):2)
            {
               k1 = k + (soc - 2)
-              #y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
+              #y = deSolve::ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1[1:3],k1,ddep),rtol = reltol,atol = abstol, method = methode)
               #probs = y[2,2:(lx+2)]
               probs = dd_loglik_M_bw(pars1[1:3],lx,k1,ddep,tt = abs(brts[k - 1] - brts[k]),probs[1:lx])
               probs = c(probs,0)
@@ -490,9 +552,9 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
              #probsn = rep(0,lx + 1)
              #probsn[S + missnumspec + 1] = 1
              #TT = max(1,1/abs(la - mu),1/abs(la2 - mu2)) * 1E+10 * max(abs(brts)) # make this more efficient later
-             #y = ode(probsn,c(0,-tshift),dd_loglik_bw_rhs,c(pars1[4:6],0,ddep),rtol = reltol,atol = abstol, method = methode)
+             #y = deSolve::ode(probsn,c(0,-tshift),dd_loglik_bw_rhs,c(pars1[4:6],0,ddep),rtol = reltol,atol = abstol, method = methode)
              #probsn = y[2,2:(lx+2)]
-             #y = ode(probsn,c(-tshift,TT),dd_loglik_bw_rhs,c(pars1[1:3],0,ddep),rtol = reltol,atol = abstol, method = methode)
+             #y = deSolve::ode(probsn,c(-tshift,TT),dd_loglik_bw_rhs,c(pars1[1:3],0,ddep),rtol = reltol,atol = abstol, method = methode)
              #logliknorm = log(y[2,lx + 2])
                        
              probsn = rep(0,lx + 1)
@@ -510,7 +572,7 @@ if(((pars1[2] == 0 || pars1[4] == 0) && (ddep == 2 | ddep == 2.1 | ddep == 2.2))
                 #probsn = rep(0,lx + 1)
                 #probsn[1:lx] = probs[1:lx]
                 #probsn = c(flavec(ddep,la,mu,K,0,lx,1,n0),1) * probsn # speciation event
-                #y = ode(probsn,c(max(abs(brts)),TT),dd_loglik_bw_rhs,c(pars1[1:3],1,ddep),rtol = reltol,atol = abstol, method = methode)
+                #y = deSolve::ode(probsn,c(max(abs(brts)),TT),dd_loglik_bw_rhs,c(pars1[1:3],1,ddep),rtol = reltol,atol = abstol, method = methode)
                 #logliknorm = logliknorm - log(y[2,lx + 2])
                 #print(log(y[2,lx + 2]))
                 probsn2 = rep(0,lx)
@@ -530,7 +592,7 @@ if(pars2[5] == 1)
     s1 = sprintf('Parameters: %f %f %f %f %f %f %f, ',pars1[1],pars1[2],pars1[3],pars1[4],pars1[5],pars1[6],pars1[7])
     s2 = sprintf('Loglikelihood: %f',loglik)
     cat(s1,s2,"\n",sep = "")
-    flush.console()
+    utils::flush.console()
 }
 loglik = as.numeric(loglik)
 if(is.nan(loglik) | is.na(loglik))
@@ -538,4 +600,4 @@ if(is.nan(loglik) | is.na(loglik))
     loglik = -Inf
 }
 return(loglik)
-}
\ No newline at end of file
+}
