@@ -42,6 +42,70 @@ dd_loglik_test = function(pars1,pars2,brts,missnumspec,methode = 'analytical',rh
   return(out)
 }
 
+
+
+#' Loglikelihood for diversity-dependent diversification models
+#' 
+#' This function computes loglikelihood of a diversity-dependent
+#' diversification model for a given set of branching times and parameter
+#' values.
+#' 
+#' 
+#' @param pars1 Vector of parameters: \cr \cr \code{pars1[1]} corresponds to
+#' lambda (speciation rate) \cr \code{pars1[2]} corresponds to mu (extinction
+#' rate) \cr \code{pars1[3]} corresponds to K (clade-level carrying capacity)
+#' @param pars2 Vector of model settings: \cr \cr \code{pars2[1]} sets the
+#' maximum number of species for which a probability must be computed.  This
+#' must be larger than 1 + missnumspec + length(brts). \cr \cr \code{pars2[2]}
+#' sets the model of diversity-dependence: \cr - \code{pars2[2] == 1} linear
+#' dependence in speciation rate with parameter K (= diversity where speciation
+#' = extinction)\cr - \code{pars2[2] == 1.3} linear dependence in speciation
+#' rate with parameter K' (= diversity where speciation = 0)\cr -
+#' \code{pars2[2] == 1.4} : positive diversity-dependence in speciation rate
+#' with parameter K' (= diversity where speciation rate reaches half its
+#' maximum); lambda = lambda0 * S/(S + K') where S is species richness\cr -
+#' \code{pars2[2] == 1.5} : positive and negative diversity-dependence in
+#' speciation rate with parameter K' (= diversity where speciation = 0); lambda
+#' = lambda0 * S/K' * (1 - S/K') where S is species richness\cr -
+#' \code{pars2[2] == 2} exponential dependence in speciation rate with
+#' parameter K (= diversity where speciation = extinction)\cr - \code{pars2[2]
+#' == 2.1} variant of exponential dependence in speciation rate with offset at
+#' infinity\cr - \code{pars2[2] == 2.2} 1/n dependence in speciation rate\cr -
+#' \code{pars2[2] == 2.3} exponential dependence in speciation rate with
+#' parameter x (= exponent)\cr - \code{pars2[2] == 3} linear dependence in
+#' extinction rate \cr - \code{pars2[2] == 4} exponential dependence in
+#' extinction rate \cr - \code{pars2[2] == 4.1} variant of exponential
+#' dependence in extinction rate with offset at infinity\cr - \code{pars2[2] ==
+#' 4.2} 1/n dependence in extinction rate\cr - \code{pars2[2] == 5} linear
+#' dependence in speciation and extinction rate \cr \cr \code{pars2[3]} sets
+#' the conditioning: \cr - \code{pars2[3] == 0} conditioning on stem or crown
+#' age \cr - \code{pars2[3] == 1} conditioning on stem or crown age and
+#' non-extinction of the phylogeny \cr - \code{pars2[3] == 2} conditioning on
+#' stem or crown age and on the total number of extant taxa (including missing
+#' species) \cr - \code{pars2[3] == 3} conditioning on the total number of
+#' extant taxa (including missing species) \cr \cr \code{pars2[4]} sets whether
+#' the likelihood is for the branching times (0) or the phylogeny (1) \cr \cr
+#' \code{pars2[5]} sets whether the parameters and likelihood should be shown
+#' on screen (1) or not (0)\cr \cr \code{pars2[6]} sets whether the first data
+#' point is stem age (1) or crown age (2)
+#' @param brts A set of branching times of a phylogeny, all positive
+#' @param missnumspec The number of species that are in the clade but missing
+#' in the phylogeny
+#' @param methode The method used to solve the master equation, default is
+#' 'analytical' which uses matrix exponentiation; alternatively numerical ODE
+#' solvers can be used, such as 'lsoda' or 'ode45'. These were used in the
+#' package before version 3.1.
+#' @return The loglikelihood
+#' @author Rampal S. Etienne & Bart Haegeman
+#' @seealso \code{\link{dd_ML}}, \code{\link{dd_SR_loglik}},
+#' \code{\link{dd_KI_loglik}}
+#' @references - Etienne, R.S. et al. 2012, Proc. Roy. Soc. B 279: 1300-1309,
+#' doi: 10.1098/rspb.2011.1439 \cr - Etienne, R.S. & B. Haegeman 2012. Am. Nat.
+#' 180: E75-E89, doi: 10.1086/667574
+#' @keywords models
+#' @examples
+#' dd_loglik(pars1 = c(0.5,0.1,100), pars2 = c(100,1,1,1,0,2), brts = 1:10, missnumspec = 0) 
+#' @export dd_loglik
 dd_loglik = function(pars1,pars2,brts,missnumspec,methode = 'analytical')
 {
    if(pars2[3] == 3)
@@ -123,6 +187,12 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
              k1 = k + (soc - 2)
              y = dd_integrate(probs,brts[(k-1):k],rhs_func_name,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
              probs = y[2,2:(lx+1)]
+             if(is.na(sum(probs)) && pars1[2]/pars1[1] < 1E-4 && missnumspec == 0)
+             {
+               loglik = dd_loglik_high_lambda(pars1 = pars1,pars2 = pars2,brts = brts)
+               warning('High lambda approximation has been applied.')
+               return(loglik)
+             }
              if(k < (S + 2 - soc))
              {
                  probs = flavec(ddep,la,mu,K,r,lx,k1,n0) * probs # speciation event
@@ -206,7 +276,7 @@ if(pars2[5] == 1)
     if(ddep == 5) {s1 = sprintf('%s %f',s1,pars1[4])}
     s2 = sprintf(', Loglikelihood: %f',loglik)
     cat(s1,s2,"\n",sep = "")
-    flush.console()
+    utils::flush.console()
 }
 }
 loglik = as.numeric(loglik)
@@ -285,11 +355,15 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
           for(k in 2:(S + 2 - soc))
           {
              k1 = k + (soc - 2)
-             #y = ode(probs,brts[(k-1):k],rhs_func,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
+             #y = deSolve::ode(probs,brts[(k-1):k],rhs_func,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
              #probs2 = y[2,2:(lx+1)]
              probs = dd_loglik_M(pars1,lx,k1,ddep,tt = abs(brts[k] - brts[k-1]),probs)
-             #print(as.numeric(probs[1:10]))
-             #print(as.numeric(probs2[1:10])) 
+             if(is.na(sum(probs)) && pars1[2]/pars1[1] < 1E-4 && missnumspec == 0)
+             {
+               loglik = dd_loglik_high_lambda(pars1 = pars1,pars2 = pars2,brts = brts)
+               warning('High lambda approximation has been applied.')
+               return(loglik)
+             }
              if(k < (S + 2 - soc))
              {
                  #probs = flavec(ddep,la,mu,K,r,lx,k1,n0) * probs # speciation event
@@ -310,7 +384,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
           for(k in (S + 2 - soc):2)
           {
              k1 = k + (soc - 2)
-             #y = ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
+             #y = deSolve::ode(probs,-brts[k:(k-1)],dd_loglik_bw_rhs,c(pars1,k1,ddep),rtol = reltol,atol = abstol,method = methode)
              #probs2 = y[2,2:(lx+2)]
              probs = dd_loglik_M_bw(pars1,lx,k1,ddep,tt = abs(brts[k] - brts[k-1]),probs[1:lx])
              probs = c(probs,0)
@@ -343,7 +417,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
              k = soc
              t1 = brts[1] 
              t2 = brts[S + 2 - soc]
-             #y = ode(probsn,c(t1,t2),rhs_func,c(pars1,k,ddep),rtol = reltol,atol = abstol,method = methode);
+             #y = deSolve::ode(probsn,c(t1,t2),rhs_func,c(pars1,k,ddep),rtol = reltol,atol = abstol,method = methode);
              #probsn = y[2,2:(lx+1)]
              probsn = dd_loglik_M(pars1,lx,k,ddep,tt = abs(t2 - t1),probsn)
              if(soc == 1) { aux = 1:lx }
@@ -357,7 +431,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
              #probsn = rep(0,lx + 1)
              #probsn[S + missnumspec + 1] = 1 #/ (S + missnumspec)
              #TT = max(1,1/abs(la - mu)) * 100000000 * max(abs(brts)) # make this more efficient later
-             #y = ode(probsn,c(0,TT),dd_loglik_bw_rhs,c(pars1,0,ddep),rtol = reltol,atol = abstol,method = methode)
+             #y = deSolve::ode(probsn,c(0,TT),dd_loglik_bw_rhs,c(pars1,0,ddep),rtol = reltol,atol = abstol,method = methode)
              #logliknorm = log(y[2,lx + 2])
              probsn = rep(0,lx + 1)
              probsn[2] = 1
@@ -373,7 +447,7 @@ if((mu == 0 & (ddep == 2 | ddep == 2.1 | ddep == 2.2)) | (la == 0 & (ddep == 4 |
                 #probsn[1:lx] = probs[1:lx]
                 #probsn = c(flavec(ddep,la,mu,K,r,lx,1,n0),1) * probsn # speciation event
                 #probsn = c(lambdamu(0:(lx - 1) + 1,pars1,ddep)[[1]],1) * probsn # speciation event
-                #y = ode(probsn,c(max(abs(brts)),TT),dd_loglik_bw_rhs,c(pars1,1,ddep),rtol = reltol,atol = abstol,method = methode)
+                #y = deSolve::ode(probsn,c(max(abs(brts)),TT),dd_loglik_bw_rhs,c(pars1,1,ddep),rtol = reltol,atol = abstol,method = methode)
                 #logliknorm = logliknorm - log(y[2,lx + 2])
                 probsn2 = rep(0,lx)
                 probsn2 = lambdamu(0:(lx - 1) + 1,pars1,ddep)[[1]] * probs[1:lx]
@@ -394,7 +468,7 @@ if(pars2[5] == 1)
     if(ddep == 5) {s1 = sprintf('%s %f',s1,pars1[4])}
     s2 = sprintf(', Loglikelihood: %f',loglik)
     cat(s1,s2,"\n",sep = "")
-    flush.console()
+    utils::flush.console()
 }
 }
 loglik = as.numeric(loglik)
@@ -430,14 +504,15 @@ dd_integrate = function(initprobs,tvec,rhs_func,pars,rtol,atol,method)
   } else
   if(rhs_func_name == 'dd_loglik_bw_rhs_FORTRAN')
   {
-    y = dd_ode_FORTRAN(initprobs,tvec,parsvec,atol,rtol,method,runmod = "runmodbw")
+    y = dd_ode_FORTRAN(initprobs,tvec,parsvec,atol,rtol,method,runmod = "dd_runmodbw")
   } else
   {
-    y = ode(initprobs,tvec,rhs_func,parsvec,rtol = rtol,atol = atol,method = method)
+    y = deSolve::ode(initprobs,tvec,rhs_func,parsvec,rtol = rtol,atol = atol,method = method)
   }
   return(y)
 }
 
+#' @useDynLib DDD
 dd_ode_FORTRAN <- function(
   initprobs,
   tvec,
@@ -445,14 +520,14 @@ dd_ode_FORTRAN <- function(
   atol,
   rtol,
   methode,
-  runmod = "runmod"
+  runmod = "dd_runmod"
 )
 {
   #system("R CMD SHLIB d:/data/ms/DDD/dd_loglik_rhs_FORTRAN.f")
   #dyn.load(paste("d:/data/ms/DDD/dd_loglik_rhs_FORTRAN", .Platform$dynlib.ext, sep = ""))
   N <- length(initprobs)
-  probs <- ode(y = initprobs, parms = c(N + 0.,parsvec[length(parsvec)] + 0.), rpar = parsvec[-length(parsvec)], 
-               times = tvec, func = runmod, initfunc = "initmod", 
+  probs <- deSolve::ode(y = initprobs, parms = c(N + 0.,parsvec[length(parsvec)] + 0.), rpar = parsvec[-length(parsvec)], 
+               times = tvec, func = runmod, initfunc = "dd_initmod", 
                ynames = c("SV"), dimens = N + 2, nout = 1, outnames = c("Sum"), 
                dllname = "DDD",atol = atol, rtol = rtol, method = methode)[,1:(N + 1)]
   #dyn.unload(paste("d:/data/ms/DDD/dd_loglik_rhs_FORTRAN", .Platform$dynlib.ext, sep = ""))
